LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY minicpu IS
	PORT (
		Data:			IN	STD_LOGIC_VECTOR(7 DOWNTO 0);
		opcode:			IN	STD_LOGIC_VECTOR(1 DOWNTO 0);
		Start, Clock:	IN	STD_LOGIC;
		BusWires:		INOUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END minicpu;

ARCHITECTURE a OF minicpu IS
	TYPE estados IS (S,L1,L2,L3,A1,A2,A3,B1,B2,B3,C1,C2,C3);
	SIGNAL estado : 	estados := S;
	SIGNAL Rin, Rout: 	STD_LOGIC_VECTOR(1 TO 3) := (OTHERS => '0');
	SIGNAL R1, R2, R3: 	STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL Extern: 	STD_LOGIC := '0';
	
	component registrador
		GENERIC (N: INTEGER := 8);
		port (
			R: in std_logic_vector(N-1 downto 0);
			Rin, Clock: IN std_logic;
			Q:	OUT std_logic_vector(N-1 downto 0)
		);
	end component;
	
	component trin
		GENERIC (N: INTEGER := 8);
		port (
			registrador:	IN STD_LOGIC_VECTOR(N-1 DOWNTO 0);
			chave:			IN STD_LOGIC;
			barramento:		OUT STD_LOGIC_VECTOR(N-1 DOWNTO 0)
		);
	end component;

BEGIN
	tri_ext: trin PORT MAP (Data, Extern, BusWires);
	reg1: registrador PORT MAP (BusWires, Rin(1), Clock, R1);
	reg2: registrador PORT MAP (BusWires, Rin(2), Clock, R2);
	reg3: registrador PORT MAP (BusWires, Rin(3), Clock, R3);
	tri1: trin PORT MAP (R1, Rout(1), BusWires);
	tri2: trin PORT MAP (R2, Rout(2), BusWires);
	tri3: trin PORT MAP (R3, Rout(3), BusWires);
	
	
	
	PROCESS(Clock, opcode)
	BEGIN
		IF Clock'EVENT AND Clock = '1' THEN
			CASE (estado) IS
			
				-- estado inicial - verifica se foi dado o start, se sim, verifica tambm o opcode
				WHEN S =>
					IF Start = '1' THEN
						CASE (opcode) IS
							WHEN "00" => --- troca R1 com R2 - R3 como auxiliar
								-- copia R1 para R3
								Rin(3) <= '1';
								Rout(1) <= '1';
								estado <= A1;
								
							WHEN "01" => --- troca R1 com R3 - R2 como auxiliar
								-- copia R1 para R2
								Rin(2) <= '1';
								Rout(1) <= '1';
								estado <= B1;
								
							WHEN "10" => --- troca R2 com R3 - R1 como auxiliar
								-- copia R2 para R1
								Rin(1) <= '1';
								Rout(2) <= '1';
								estado <= C1;
							
							--carrega os dados nos registradores
							WHEN "11" => --- ativa R1in e Extern, para que o barramento receba o valor de Data e R1 acesse esse valor
								Extern <= '1';
								Rin(1) <= '1';
								estado <= L1;
						END CASE;
					END IF;

				-- ativa R2in para que R2 acesse o barramento
				WHEN L1 =>
					Rin(1) <= '0';
					Rin(2) <= '1';
					estado <= L2;
				
				-- ativa R3in para que R3 acesse o barramento
				WHEN L2 =>
					Rin(2) <= '0';
					Rin(3) <= '1';
					estado <= L3;
				
				-- desativa o Extern
				WHEN L3 =>
					Rin(3) <= '0';
					estado <= S;
					Extern <= '0';
				
				-- troca R1 com R2 - R3 como auxiliar
				WHEN A1 => -- copia R1 para R2
					Rin(3) <= '0';
					Rout(1) <= '0';
					Rin(1) <= '1';
					Rout(2) <= '1';
					estado <= A2;

				WHEN A2 => -- copia R3 para R2
					Rin(1) <= '0';
					Rout(2) <= '0';
					Rin(2) <= '1';
					Rout(3) <= '1';
					
				WHEN A3 => -- finaliza
					Rout(3) <= '0';
					Rin(2) <= '0';
					estado <= S;
					
				WHEN B1 => -- copia R3 para R1
					Rin(2) <= '0';
					Rout(1) <= '0';
					Rin(1) <= '1';
					Rout(3) <= '1';
					estado <= B2;
					
				WHEN B2 => -- copia R2 para R3
					Rin(1) <= '0';
					Rout(3) <= '0';
					Rin(3) <= '1';
					Rout(2) <= '1';
					estado <= B3;
					
				WHEN B3 => -- finaliza
					Rin(3) <= '0';
					Rout(2) <= '0';
					estado <= S;
				
				WHEN C1 => -- copia R3 para R2
					Rin(1) <= '0';
					Rout(2) <= '0';
					Rin(2) <= '1';
					Rout(3) <= '1';
					estado <= C2;
					
				WHEN C2 => -- copia R1 para R3
					Rin(2) <= '0';
					Rout(3) <= '0';
					Rin(3) <= '1';
					Rout(1) <= '1';
					estado <= C3;
					
				WHEN C3 => -- finaliza
					Rin(3) <= '0';
					Rout(1) <= '0';
					estado <= S;
			END CASE;
		END IF;
	END PROCESS;
		
END a;